@@ -6,8 +6,9 @@ import threading
 import time
 import types
 import warnings
-from contextlib import contextmanager
+from contextlib import contextmanager, ExitStack
 from math import inf
+from textwrap import dedent
 
 import attr
 import outcome
@@ -15,9 +16,9 @@ import sniffio
 import pytest
 from async_generator import async_generator
 
-from .tutil import check_sequence_matches, gc_collect_harder
+from .tutil import slow, check_sequence_matches, gc_collect_harder
 from ... import _core
-from ..._threads import run_sync_in_worker_thread
+from ..._threads import to_thread_run_sync
 from ..._timeouts import sleep, fail_after
 from ..._util import aiter_compat
 from ...testing import (
@@ -190,8 +191,8 @@ def test_main_and_task_both_crash():
     with pytest.raises(_core.MultiError) as excinfo:
         _core.run(main)
     print(excinfo.value)
-    assert set(type(exc)
-               for exc in excinfo.value.exceptions) == {ValueError, KeyError}
+    assert {type(exc)
+            for exc in excinfo.value.exceptions} == {ValueError, KeyError}
 
 
 def test_two_child_crashes():
@@ -205,8 +206,8 @@ def test_two_child_crashes():
 
     with pytest.raises(_core.MultiError) as excinfo:
         _core.run(main)
-    assert set(type(exc)
-               for exc in excinfo.value.exceptions) == {ValueError, KeyError}
+    assert {type(exc)
+            for exc in excinfo.value.exceptions} == {ValueError, KeyError}
 
 
 async def test_child_crash_wakes_parent():
@@ -346,9 +347,9 @@ async def test_current_statistics(mock_clock):
     assert stats.seconds_to_next_deadline == inf
 
 
-@attr.s(cmp=False, hash=False)
+@attr.s(eq=False, hash=False)
 class TaskRecorder:
-    record = attr.ib(default=attr.Factory(list))
+    record = attr.ib(factory=list)
 
     def before_run(self):
         self.record.append(("before_run",))
@@ -411,14 +412,9 @@ def test_instruments(recwarn):
     # reschedules the task immediately upon yielding, before the
     # after_task_step event fires.
     expected = (
-        [("before_run",),
-         ("schedule", task)] +
-        [("before", task),
-         ("schedule", task),
-         ("after", task)] * 5 + [
-             ("before", task),
-             ("after", task), ("after_run",)
-         ]
+        [("before_run",), ("schedule", task)] +
+        [("before", task), ("schedule", task), ("after", task)] * 5 +
+        [("before", task), ("after", task), ("after_run",)]
     )
     assert len(r1.record) > len(r2.record) > len(r3.record)
     assert r1.record == r2.record + r3.record
@@ -551,17 +547,13 @@ async def test_cancel_scope_repr(mock_clock):
     scope = _core.CancelScope()
     assert "unbound" in repr(scope)
     with scope:
-        assert "bound to 1 task" in repr(scope)
-        async with _core.open_nursery() as nursery:
-            nursery.start_soon(sleep, 10)
-            assert "bound to 2 tasks" in repr(scope)
-            nursery.cancel_scope.cancel()
+        assert "active" in repr(scope)
         scope.deadline = _core.current_time() - 1
         assert "deadline is 1.00 seconds ago" in repr(scope)
         scope.deadline = _core.current_time() + 10
         assert "deadline is 10.00 seconds from now" in repr(scope)
         # when not in async context, can't get the current time
-        assert "deadline" not in await run_sync_in_worker_thread(repr, scope)
+        assert "deadline" not in await to_thread_run_sync(repr, scope)
         scope.cancel()
         assert "cancelled" in repr(scope)
     assert "exited" in repr(scope)
@@ -813,13 +805,6 @@ async def test_basic_timeout(mock_clock):
             await _core.checkpoint()
 
 
-@pytest.mark.filterwarnings(
-    "ignore:.*trio.open_cancel_scope:trio.TrioDeprecationWarning"
-)
-async def test_cancel_scope_deprecated(recwarn):
-    assert isinstance(_core.open_cancel_scope(), _core.CancelScope)
-
-
 async def test_cancel_scope_nesting():
     # Nested scopes: if two triggering at once, the outer one wins
     with _core.CancelScope() as scope1:
@@ -858,6 +843,18 @@ async def test_cancel_scope_nesting():
     assert not scope.cancelled_caught
 
 
+# Regression test for https://github.com/python-trio/trio/issues/1175
+async def test_unshield_while_cancel_propagating():
+    with _core.CancelScope() as outer:
+        with _core.CancelScope() as inner:
+            outer.cancel()
+            try:
+                await _core.checkpoint()
+            finally:
+                inner.shield = True
+    assert outer.cancelled_caught and not inner.cancelled_caught
+
+
 async def test_cancel_unbound():
     async def sleep_until_cancelled(scope):
         with scope, fail_after(1):
@@ -876,6 +873,16 @@ async def test_cancel_unbound():
         await wait_all_tasks_blocked()
         scope.cancel()
 
+    # Shield before entry
+    scope = _core.CancelScope()
+    scope.shield = True
+    with _core.CancelScope() as outer, scope:
+        outer.cancel()
+        await _core.checkpoint()
+        scope.shield = False
+        with pytest.raises(_core.Cancelled):
+            await _core.checkpoint()
+
     # Can't reuse
     with _core.CancelScope() as scope:
         await _core.checkpoint()
@@ -912,10 +919,90 @@ async def test_cancel_unbound():
         assert "single 'with' block" in str(exc_info.value)
         nursery.cancel_scope.cancel()
 
+    # If not yet entered, cancel_called is true when the deadline has passed
+    # even if cancel() hasn't been called yet
+    scope = _core.CancelScope(deadline=_core.current_time() + 1)
+    assert not scope.cancel_called
+    scope.deadline -= 1
+    assert scope.cancel_called
+    scope.deadline += 1
+    assert scope.cancel_called  # never become un-cancelled
 
+
+async def test_cancel_scope_misnesting():
+    outer = _core.CancelScope()
+    inner = _core.CancelScope()
+    with ExitStack() as stack:
+        stack.enter_context(outer)
+        with inner:
+            with pytest.raises(RuntimeError, match="still within its child"):
+                stack.close()
+        # No further error is raised when exiting the inner context
+
+    # If there are other tasks inside the abandoned part of the cancel tree,
+    # they get cancelled when the misnesting is detected
+    async def task1():
+        with pytest.raises(_core.Cancelled):
+            await sleep_forever()
+
+    # Even if inside another cancel scope
+    async def task2():
+        with _core.CancelScope():
+            with pytest.raises(_core.Cancelled):
+                await sleep_forever()
+
+    with ExitStack() as stack:
+        stack.enter_context(_core.CancelScope())
+        async with _core.open_nursery() as nursery:
+            nursery.start_soon(task1)
+            nursery.start_soon(task2)
+            await wait_all_tasks_blocked()
+            with pytest.raises(RuntimeError, match="still within its child"):
+                stack.close()
+
+    # Variant that makes the child tasks direct children of the scope
+    # that noticed the misnesting:
+    nursery_mgr = _core.open_nursery()
+    nursery = await nursery_mgr.__aenter__()
+    try:
+        nursery.start_soon(task1)
+        nursery.start_soon(task2)
+        nursery.start_soon(sleep_forever)
+        await wait_all_tasks_blocked()
+        nursery.cancel_scope.__exit__(None, None, None)
+    finally:
+        with pytest.raises(RuntimeError) as exc_info:
+            await nursery_mgr.__aexit__(*sys.exc_info())
+        assert "which had already been exited" in str(exc_info.value)
+        assert type(exc_info.value.__context__) is _core.MultiError
+        assert len(exc_info.value.__context__.exceptions) == 3
+        cancelled_in_context = False
+        for exc in exc_info.value.__context__.exceptions:
+            assert isinstance(exc, RuntimeError)
+            assert "closed before the task exited" in str(exc)
+            cancelled_in_context |= isinstance(
+                exc.__context__, _core.Cancelled
+            )
+        assert cancelled_in_context  # for the sleep_forever
+
+    # Trying to exit a cancel scope from an unrelated task raises an error
+    # without affecting any state
+    async def task3(task_status):
+        with _core.CancelScope() as scope:
+            task_status.started(scope)
+            await sleep_forever()
+
+    async with _core.open_nursery() as nursery:
+        scope = await nursery.start(task3)
+        with pytest.raises(RuntimeError, match="from unrelated"):
+            scope.__exit__(None, None, None)
+        scope.cancel()
+
+
+@slow
 async def test_timekeeping():
     # probably a good idea to use a real clock for *one* test anyway...
-    TARGET = 0.1
+    TARGET = 1.0
     # give it a few tries in case of random CI server flakiness
     for _ in range(4):
         real_start = time.perf_counter()
@@ -1112,7 +1199,7 @@ async def test_yield_briefly_checks_for_timeout(mock_clock):
 
 # This tests that sys.exc_info is properly saved/restored as we swap between
 # tasks. It turns out that the interpreter automagically handles this for us
-# so there's no special code in trio required to pass this test, but it's
+# so there's no special code in Trio required to pass this test, but it's
 # still nice to know that it works :-).
 #
 # Update: it turns out I was right to be nervous! see the next test...
@@ -1619,6 +1706,8 @@ async def test_current_effective_deadline(mock_clock):
     assert _core.current_effective_deadline() == inf
 
 
+# @coroutine is deprecated since python 3.8, which is fine with us.
+@pytest.mark.filterwarnings("ignore:.*@coroutine.*:DeprecationWarning")
 def test_nice_error_on_bad_calls_to_run_or_spawn():
     def bad_call_run(*args):
         _core.run(*args)
@@ -1657,10 +1746,6 @@ def test_nice_error_on_bad_calls_to_run_or_spawn():
                 bad_call(asyncio.Future())
             assert "asyncio" in str(excinfo.value)
 
-            with pytest.raises(TypeError) as excinfo:
-                bad_call(generator_based_coro)
-            assert "asyncio" in str(excinfo.value)
-
             with pytest.raises(TypeError) as excinfo:
                 bad_call(lambda: asyncio.Future())
             assert "asyncio" in str(excinfo.value)
@@ -1691,14 +1776,31 @@ def test_nice_error_on_bad_calls_to_run_or_spawn():
 
 
 def test_calling_asyncio_function_gives_nice_error():
-    async def misguided():
+    async def child_xyzzy():
         import asyncio
         await asyncio.Future()
 
+    async def misguided():
+        await child_xyzzy()
+
     with pytest.raises(TypeError) as excinfo:
         _core.run(misguided)
 
     assert "asyncio" in str(excinfo.value)
+    # The traceback should point to the location of the foreign await
+    assert any(  # pragma: no branch
+        entry.name == "child_xyzzy" for entry in excinfo.traceback
+    )
+
+
+async def test_asyncio_function_inside_nursery_does_not_explode():
+    # Regression test for https://github.com/python-trio/trio/issues/552
+    with pytest.raises(TypeError) as excinfo:
+        async with _core.open_nursery() as nursery:
+            import asyncio
+            nursery.start_soon(sleep_forever)
+            await asyncio.Future()
+    assert "asyncio" in str(excinfo.value)
 
 
 async def test_trivial_yields():
@@ -1719,9 +1821,10 @@ async def test_trivial_yields():
             async with _core.open_nursery():
                 raise KeyError
         assert len(excinfo.value.exceptions) == 2
-        assert set(type(e) for e in excinfo.value.exceptions) == {
-            KeyError, _core.Cancelled
-        }
+        assert {type(e)
+                for e in excinfo.value.exceptions} == {
+                    KeyError, _core.Cancelled
+                }
 
 
 async def test_nursery_start(autojump_clock):
@@ -1810,9 +1913,8 @@ async def test_nursery_start(autojump_clock):
             cs.cancel()
             with pytest.raises(_core.MultiError) as excinfo:
                 await nursery.start(raise_keyerror_after_started)
-    assert set(type(e) for e in excinfo.value.exceptions) == {
-        _core.Cancelled, KeyError
-    }
+    assert {type(e)
+            for e in excinfo.value.exceptions} == {_core.Cancelled, KeyError}
 
     # trying to start in a closed nursery raises an error immediately
     async with _core.open_nursery() as closed_nursery:
@@ -1927,7 +2029,7 @@ async def test_nursery_stop_iteration():
 
 
 async def test_nursery_stop_async_iteration():
-    class it(object):
+    class it:
         def __init__(self, count):
             self.count = count
             self.val = 0
@@ -1940,7 +2042,7 @@ async def test_nursery_stop_async_iteration():
             self.val += 1
             return val
 
-    class async_zip(object):
+    class async_zip:
         def __init__(self, *largs):
             self.nexts = [obj.__anext__ for obj in largs]
 
@@ -2064,9 +2166,33 @@ def test_system_task_contexts():
     _core.run(inner)
 
 
+def test_Nursery_init():
+    check_Nursery_error = pytest.raises(
+        TypeError, match='no public constructor available'
+    )
+
+    with check_Nursery_error:
+        _core._run.Nursery(None, None)
+
+
+async def test_Nursery_private_init():
+    # context manager creation should not raise
+    async with _core.open_nursery() as nursery:
+        assert False == nursery._closed
+
+
+def test_Nursery_subclass():
+    with pytest.raises(
+        TypeError, match='`Nursery` does not support subclassing'
+    ):
+
+        class Subclass(_core._run.Nursery):
+            pass
+
+
 def test_Cancelled_init():
     check_Cancelled_error = pytest.raises(
-        RuntimeError, match='should not be raised directly'
+        TypeError, match='no public constructor available'
     )
 
     with check_Cancelled_error:
@@ -2076,7 +2202,30 @@ def test_Cancelled_init():
         _core.Cancelled()
 
     # private constructor should not raise
-    _core.Cancelled._init()
+    _core.Cancelled._create()
+
+
+def test_Cancelled_str():
+    cancelled = _core.Cancelled._create()
+    assert str(cancelled) == 'Cancelled'
+
+
+def test_Cancelled_subclass():
+    with pytest.raises(
+        TypeError, match='`Cancelled` does not support subclassing'
+    ):
+
+        class Subclass(_core.Cancelled):
+            pass
+
+
+def test_CancelScope_subclass():
+    with pytest.raises(
+        TypeError, match='`CancelScope` does not support subclassing'
+    ):
+
+        class Subclass(_core.CancelScope):
+            pass
 
 
 def test_sniffio_integration():
@@ -2234,3 +2383,50 @@ async def test_detached_coroutine_cancellation():
         task.coro.send(None)
 
     assert abort_fn_called
+
+
+def test_async_function_implemented_in_C():
+    # These used to crash because we'd try to mutate the coroutine object's
+    # cr_frame, but C functions don't have Python frames.
+
+    ns = {"_core": _core}
+    try:
+        exec(
+            dedent(
+                """
+                async def agen_fn(record):
+                    assert not _core.currently_ki_protected()
+                    record.append("the generator ran")
+                    yield
+                """
+            ),
+            ns,
+        )
+    except SyntaxError:
+        pytest.skip("Requires Python 3.6+")
+    else:
+        agen_fn = ns["agen_fn"]
+
+    run_record = []
+    agen = agen_fn(run_record)
+    _core.run(agen.__anext__)
+    assert run_record == ["the generator ran"]
+
+    async def main():
+        start_soon_record = []
+        agen = agen_fn(start_soon_record)
+        async with _core.open_nursery() as nursery:
+            nursery.start_soon(agen.__anext__)
+        assert start_soon_record == ["the generator ran"]
+
+    _core.run(main)
+
+
+async def test_very_deep_cancel_scope_nesting():
+    # This used to crash with a RecursionError in CancelStatus.recalculate
+    with ExitStack() as exit_stack:
+        outermost_scope = _core.CancelScope()
+        exit_stack.enter_context(outermost_scope)
+        for _ in range(5000):
+            exit_stack.enter_context(_core.CancelScope())
+        outermost_scope.cancel()
