@@ -1,12 +1,13 @@
-import operator
-from collections import deque, OrderedDict
 import math
 
 import attr
 import outcome
 
-from . import _core
+import trio
+
 from ._util import aiter_compat
+from ._core import enable_ki_protection, ParkingLot
+from ._deprecate import deprecated
 
 __all__ = [
     "Event",
@@ -18,7 +19,7 @@ __all__ = [
 ]
 
 
-@attr.s(repr=False, cmp=False, hash=False)
+@attr.s(repr=False, eq=False, hash=False)
 class Event:
     """A waitable boolean value useful for inter-task synchronization,
     inspired by :class:`threading.Event`.
@@ -36,9 +37,16 @@ class Event:
     primitive that doesn't have this protection, consider :class:`Condition`
     or :class:`trio.hazmat.ParkingLot`.
 
+    .. note:: Unlike `threading.Event`, `trio.Event` has no
+       `~threading.Event.clear` method. In Trio, once an `Event` has happened,
+       it cannot un-happen. If you need to represent a series of events,
+       consider creating a new `Event` object for each one (they're cheap!),
+       or other synchronization methods like :ref:`channels <channels>` or
+       `trio.hazmat.ParkingLot`.
+
     """
 
-    _lot = attr.ib(default=attr.Factory(_core.ParkingLot), init=False)
+    _lot = attr.ib(factory=ParkingLot, init=False)
     _flag = attr.ib(default=False, init=False)
 
     def is_set(self):
@@ -47,7 +55,7 @@ class Event:
         """
         return self._flag
 
-    @_core.enable_ki_protection
+    @enable_ki_protection
     def set(self):
         """Set the internal flag value to True, and wake any waiting tasks.
 
@@ -55,10 +63,12 @@ class Event:
         self._flag = True
         self._lot.unpark_all()
 
+    @deprecated(
+        "0.12.0",
+        issue=637,
+        instead="multiple Event objects or other synchronization primitives"
+    )
     def clear(self):
-        """Set the internal flag value to False.
-
-        """
         self._flag = False
 
     async def wait(self):
@@ -68,7 +78,7 @@ class Event:
 
         """
         if self._flag:
-            await _core.checkpoint()
+            await trio.hazmat.checkpoint()
         else:
             await self._lot.park()
 
@@ -85,14 +95,14 @@ class Event:
 
 
 def async_cm(cls):
-    @_core.enable_ki_protection
+    @enable_ki_protection
     async def __aenter__(self):
         await self.acquire()
 
     __aenter__.__qualname__ = cls.__qualname__ + ".__aenter__"
     cls.__aenter__ = __aenter__
 
-    @_core.enable_ki_protection
+    @enable_ki_protection
     async def __aexit__(self, *args):
         self.release()
 
@@ -143,9 +153,9 @@ class CapacityLimiter:
     fixed number of seats, and if they're all taken then you have to wait for
     someone to get up before you can sit down.
 
-    By default, :func:`run_sync_in_worker_thread` uses a
+    By default, :func:`trio.to_thread.run_sync` uses a
     :class:`CapacityLimiter` to limit the number of threads running at once;
-    see :func:`current_default_worker_thread_limiter` for details.
+    see `trio.to_thread.current_default_thread_limiter` for details.
 
     If you're familiar with semaphores, then you can think of this as a
     restricted semaphore that's specialized for one common use case, with
@@ -163,9 +173,8 @@ class CapacityLimiter:
        just borrowed and then put back.
 
     """
-
     def __init__(self, total_tokens):
-        self._lot = _core.ParkingLot()
+        self._lot = ParkingLot()
         self._borrowers = set()
         # Maps tasks attempting to acquire -> borrower, to handle on-behalf-of
         self._pending_borrowers = {}
@@ -226,7 +235,7 @@ class CapacityLimiter:
         """
         return self.total_tokens - self.borrowed_tokens
 
-    @_core.enable_ki_protection
+    @enable_ki_protection
     def acquire_nowait(self):
         """Borrow a token from the sack, without blocking.
 
@@ -236,9 +245,9 @@ class CapacityLimiter:
               tokens.
 
         """
-        self.acquire_on_behalf_of_nowait(_core.current_task())
+        self.acquire_on_behalf_of_nowait(trio.hazmat.current_task())
 
-    @_core.enable_ki_protection
+    @enable_ki_protection
     def acquire_on_behalf_of_nowait(self, borrower):
         """Borrow a token from the sack on behalf of ``borrower``, without
         blocking.
@@ -246,7 +255,7 @@ class CapacityLimiter:
         Args:
           borrower: A :class:`trio.hazmat.Task` or arbitrary opaque object
              used to record who is borrowing this token. This is used by
-             :func:`run_sync_in_worker_thread` to allow threads to "hold
+             :func:`trio.to_thread.run_sync` to allow threads to "hold
              tokens", with the intention in the future of using it to `allow
              deadlock detection and other useful things
              <https://github.com/python-trio/trio/issues/182>`__
@@ -265,9 +274,9 @@ class CapacityLimiter:
         if len(self._borrowers) < self._total_tokens and not self._lot:
             self._borrowers.add(borrower)
         else:
-            raise _core.WouldBlock
+            raise trio.WouldBlock
 
-    @_core.enable_ki_protection
+    @enable_ki_protection
     async def acquire(self):
         """Borrow a token from the sack, blocking if necessary.
 
@@ -276,9 +285,9 @@ class CapacityLimiter:
               tokens.
 
         """
-        await self.acquire_on_behalf_of(_core.current_task())
+        await self.acquire_on_behalf_of(trio.hazmat.current_task())
 
-    @_core.enable_ki_protection
+    @enable_ki_protection
     async def acquire_on_behalf_of(self, borrower):
         """Borrow a token from the sack on behalf of ``borrower``, blocking if
         necessary.
@@ -293,24 +302,21 @@ class CapacityLimiter:
              tokens.
 
         """
-        await _core.checkpoint_if_cancelled()
+        await trio.hazmat.checkpoint_if_cancelled()
         try:
             self.acquire_on_behalf_of_nowait(borrower)
-        except _core.WouldBlock:
-            task = _core.current_task()
+        except trio.WouldBlock:
+            task = trio.hazmat.current_task()
             self._pending_borrowers[task] = borrower
             try:
                 await self._lot.park()
-            except _core.Cancelled:
+            except trio.Cancelled:
                 self._pending_borrowers.pop(task)
                 raise
-        except:
-            await _core.cancel_shielded_checkpoint()
-            raise
         else:
-            await _core.cancel_shielded_checkpoint()
+            await trio.hazmat.cancel_shielded_checkpoint()
 
-    @_core.enable_ki_protection
+    @enable_ki_protection
     def release(self):
         """Put a token back into the sack.
 
@@ -319,9 +325,9 @@ class CapacityLimiter:
               sack's tokens.
 
         """
-        self.release_on_behalf_of(_core.current_task())
+        self.release_on_behalf_of(trio.hazmat.current_task())
 
-    @_core.enable_ki_protection
+    @enable_ki_protection
     def release_on_behalf_of(self, borrower):
         """Put a token back into the sack on behalf of ``borrower``.
 
@@ -392,7 +398,6 @@ class Semaphore:
         ``max_value``.
 
     """
-
     def __init__(self, initial_value, *, max_value=None):
         if not isinstance(initial_value, int):
             raise TypeError("initial_value must be an int")
@@ -407,7 +412,7 @@ class Semaphore:
         # Invariants:
         # bool(self._lot) implies self._value == 0
         # (or equivalently: self._value > 0 implies not self._lot)
-        self._lot = _core.ParkingLot()
+        self._lot = trio.hazmat.ParkingLot()
         self._value = initial_value
         self._max_value = max_value
 
@@ -436,7 +441,7 @@ class Semaphore:
         """
         return self._max_value
 
-    @_core.enable_ki_protection
+    @enable_ki_protection
     def acquire_nowait(self):
         """Attempt to decrement the semaphore value, without blocking.
 
@@ -448,23 +453,23 @@ class Semaphore:
             assert not self._lot
             self._value -= 1
         else:
-            raise _core.WouldBlock
+            raise trio.WouldBlock
 
-    @_core.enable_ki_protection
+    @enable_ki_protection
     async def acquire(self):
         """Decrement the semaphore value, blocking if necessary to avoid
         letting it drop below zero.
 
         """
-        await _core.checkpoint_if_cancelled()
+        await trio.hazmat.checkpoint_if_cancelled()
         try:
             self.acquire_nowait()
-        except _core.WouldBlock:
+        except trio.WouldBlock:
             await self._lot.park()
         else:
-            await _core.cancel_shielded_checkpoint()
+            await trio.hazmat.cancel_shielded_checkpoint()
 
-    @_core.enable_ki_protection
+    @enable_ki_protection
     def release(self):
         """Increment the semaphore value, possibly waking a task blocked in
         :meth:`acquire`.
@@ -502,7 +507,7 @@ class _LockStatistics:
 
 
 @async_cm
-@attr.s(cmp=False, hash=False, repr=False)
+@attr.s(eq=False, hash=False, repr=False)
 class Lock:
     """A classic `mutex
     <https://en.wikipedia.org/wiki/Lock_(computer_science)>`__.
@@ -516,7 +521,7 @@ class Lock:
 
     """
 
-    _lot = attr.ib(default=attr.Factory(_core.ParkingLot), init=False)
+    _lot = attr.ib(factory=ParkingLot, init=False)
     _owner = attr.ib(default=None, init=False)
 
     def __repr__(self):
@@ -541,7 +546,7 @@ class Lock:
         """
         return self._owner is not None
 
-    @_core.enable_ki_protection
+    @enable_ki_protection
     def acquire_nowait(self):
         """Attempt to acquire the lock, without blocking.
 
@@ -550,32 +555,32 @@ class Lock:
 
         """
 
-        task = _core.current_task()
+        task = trio.hazmat.current_task()
         if self._owner is task:
             raise RuntimeError("attempt to re-acquire an already held Lock")
         elif self._owner is None and not self._lot:
             # No-one owns it
             self._owner = task
         else:
-            raise _core.WouldBlock
+            raise trio.WouldBlock
 
-    @_core.enable_ki_protection
+    @enable_ki_protection
     async def acquire(self):
         """Acquire the lock, blocking if necessary.
 
         """
-        await _core.checkpoint_if_cancelled()
+        await trio.hazmat.checkpoint_if_cancelled()
         try:
             self.acquire_nowait()
-        except _core.WouldBlock:
+        except trio.WouldBlock:
             # NOTE: it's important that the contended acquire path is just
             # "_lot.park()", because that's how Condition.wait() acquires the
             # lock as well.
             await self._lot.park()
         else:
-            await _core.cancel_shielded_checkpoint()
+            await trio.hazmat.cancel_shielded_checkpoint()
 
-    @_core.enable_ki_protection
+    @enable_ki_protection
     def release(self):
         """Release the lock.
 
@@ -583,7 +588,7 @@ class Lock:
           RuntimeError: if the calling task does not hold the lock.
 
         """
-        task = _core.current_task()
+        task = trio.hazmat.current_task()
         if task is not self._owner:
             raise RuntimeError("can't release a Lock you don't own")
         if self._lot:
@@ -663,7 +668,7 @@ class StrictFIFOLock(Lock):
     the same order that the state machine generated it.
 
     Currently, :class:`StrictFIFOLock` is simply an alias for :class:`Lock`,
-    but (a) this may not always be true in the future, especially if trio ever
+    but (a) this may not always be true in the future, especially if Trio ever
     implements `more sophisticated scheduling policies
     <https://github.com/python-trio/trio/issues/32>`__, and (b) the above code
     is relying on a pretty subtle property of its lock. Using a
@@ -694,14 +699,13 @@ class Condition:
           and used.
 
     """
-
     def __init__(self, lock=None):
         if lock is None:
             lock = Lock()
         if not type(lock) is Lock:
             raise TypeError("lock must be a trio.Lock")
         self._lock = lock
-        self._lot = _core.ParkingLot()
+        self._lot = trio.hazmat.ParkingLot()
 
     def locked(self):
         """Check whether the underlying lock is currently held.
@@ -733,9 +737,9 @@ class Condition:
         """
         self._lock.release()
 
-    @_core.enable_ki_protection
+    @enable_ki_protection
     async def wait(self):
-        """Wait for another thread to call :meth:`notify` or
+        """Wait for another task to call :meth:`notify` or
         :meth:`notify_all`.
 
         When calling this method, you must hold the lock. It releases the lock
@@ -757,7 +761,7 @@ class Condition:
           RuntimeError: if the calling task does not hold the lock.
 
         """
-        if _core.current_task() is not self._lock._owner:
+        if trio.hazmat.current_task() is not self._lock._owner:
             raise RuntimeError("must hold the lock to wait")
         self.release()
         # NOTE: we go to sleep on self._lot, but we'll wake up on
@@ -765,7 +769,7 @@ class Condition:
         try:
             await self._lot.park()
         except:
-            with _core.CancelScope(shield=True):
+            with trio.CancelScope(shield=True):
                 await self.acquire()
             raise
 
@@ -779,7 +783,7 @@ class Condition:
           RuntimeError: if the calling task does not hold the lock.
 
         """
-        if _core.current_task() is not self._lock._owner:
+        if trio.hazmat.current_task() is not self._lock._owner:
             raise RuntimeError("must hold the lock to notify")
         self._lot.repark(self._lock._lot, count=n)
 
@@ -790,7 +794,7 @@ class Condition:
           RuntimeError: if the calling task does not hold the lock.
 
         """
-        if _core.current_task() is not self._lock._owner:
+        if trio.hazmat.current_task() is not self._lock._owner:
             raise RuntimeError("must hold the lock to notify")
         self._lot.repark_all(self._lock._lot)
 
