@@ -154,7 +154,7 @@ def current_default_thread_limiter():
 # system; see https://github.com/python-trio/trio/issues/182
 # But for now we just need an object to stand in for the thread, so we can
 # keep track of who's holding the CapacityLimiter's token.
-@attr.s(frozen=True, cmp=False, hash=False)
+@attr.s(frozen=True, eq=False, hash=False)
 class ThreadPlaceholder:
     name = attr.ib()
 
@@ -210,7 +210,7 @@ async def to_thread_run_sync(sync_fn, *args, cancellable=False, limiter=None):
       background** – we just abandon it to do whatever it's going to do, and
       silently discard any return value or errors that it raises. Only use
       this if you know that the operation is safe and side-effect free. (For
-      example: :func:`trio.socket.getaddrinfo` is uses a thread with
+      example: :func:`trio.socket.getaddrinfo` uses a thread with
       ``cancellable=True``, because it doesn't really affect anything if a
       stray hostname lookup keeps running in the background.)
 
@@ -378,7 +378,6 @@ def from_thread_run(afn, *args, trio_token=None):
           "foreign" thread, spawned using some other framework, and still want
           to enter Trio.
     """
-
     def callback(q, afn, args):
         @disable_ki_protection
         async def unprotected_afn():
@@ -424,7 +423,6 @@ def from_thread_run_sync(fn, *args, trio_token=None):
           "foreign" thread, spawned using some other framework, and still want
           to enter Trio.
     """
-
     def callback(q, fn, args):
         @disable_ki_protection
         def unprotected_fn():
